@@ -34,11 +34,10 @@
 #include <gio/gio.h>
 #include <stdlib.h>
 
+#include "gcm-color.h"
 #include "gcm-edid.h"
 #include "gcm-tables.h"
 
-#include "egg-debug.h"
-
 static void     gcm_edid_finalize	(GObject     *object);
 
 #define GCM_EDID_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GCM_TYPE_EDID, GcmEdidPrivate))
@@ -59,6 +58,10 @@ struct _GcmEdidPrivate
 	guint				 width;
 	guint				 height;
 	gfloat				 gamma;
+	GcmColorYxy			*red;
+	GcmColorYxy			*green;
+	GcmColorYxy			*blue;
+	GcmColorYxy			*white;
 	GcmTables			*tables;
 };
 
@@ -73,6 +76,10 @@ enum {
 	PROP_PNP_ID,
 	PROP_WIDTH,
 	PROP_HEIGHT,
+	PROP_RED,
+	PROP_GREEN,
+	PROP_BLUE,
+	PROP_WHITE,
 	PROP_LAST
 };
 
@@ -92,7 +99,6 @@ G_DEFINE_TYPE (GcmEdid, gcm_edid, G_TYPE_OBJECT)
 #define GCM_DESCRIPTOR_ALPHANUMERIC_DATA_STRING		0xfe
 #define GCM_DESCRIPTOR_COLOR_POINT			0xfb
 
-
 /**
  * gcm_edid_get_monitor_name:
  * @edid: a valid #GcmEdid instance
@@ -101,7 +107,7 @@ G_DEFINE_TYPE (GcmEdid, gcm_edid, G_TYPE_OBJECT)
  *
  * Return value: a null terminaled string value
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 const gchar *
 gcm_edid_get_monitor_name (GcmEdid *edid)
@@ -118,7 +124,7 @@ gcm_edid_get_monitor_name (GcmEdid *edid)
  *
  * Return value: a null terminaled string value
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 const gchar *
 gcm_edid_get_vendor_name (GcmEdid *edid)
@@ -139,7 +145,7 @@ gcm_edid_get_vendor_name (GcmEdid *edid)
  *
  * Return value: a null terminaled string value
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 const gchar *
 gcm_edid_get_serial_number (GcmEdid *edid)
@@ -156,7 +162,7 @@ gcm_edid_get_serial_number (GcmEdid *edid)
  *
  * Return value: a null terminaled string value
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 const gchar *
 gcm_edid_get_eisa_id (GcmEdid *edid)
@@ -174,7 +180,7 @@ gcm_edid_get_eisa_id (GcmEdid *edid)
  *
  * Return value: a null terminaled string value
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 const gchar *
 gcm_edid_get_checksum (GcmEdid *edid)
@@ -191,7 +197,7 @@ gcm_edid_get_checksum (GcmEdid *edid)
  *
  * Return value: a null terminaled string value
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 const gchar *
 gcm_edid_get_pnp_id (GcmEdid *edid)
@@ -208,7 +214,7 @@ gcm_edid_get_pnp_id (GcmEdid *edid)
  *
  * Return value: the width in mm.
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 guint
 gcm_edid_get_width (GcmEdid *edid)
@@ -225,7 +231,7 @@ gcm_edid_get_width (GcmEdid *edid)
  *
  * Return value: the height in mm.
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 guint
 gcm_edid_get_height (GcmEdid *edid)
@@ -242,7 +248,7 @@ gcm_edid_get_height (GcmEdid *edid)
  *
  * Return value: the gamma value, although this is typically 2.2
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 gfloat
 gcm_edid_get_gamma (GcmEdid *edid)
@@ -251,13 +257,81 @@ gcm_edid_get_gamma (GcmEdid *edid)
 	return edid->priv->gamma;
 }
 
+/**
+ * gcm_edid_get_red:
+ * @edid: a valid #GcmEdid instance
+ *
+ * Gets the monitor red chromaticity value.
+ *
+ * Return value: the #GcmColorYxy value
+ *
+ * Since: 2.91.1
+ **/
+const GcmColorYxy *
+gcm_edid_get_red (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->red;
+}
+
+/**
+ * gcm_edid_get_green:
+ * @edid: a valid #GcmEdid instance
+ *
+ * Gets the monitor green chromaticity value.
+ *
+ * Return value: the #GcmColorYxy value
+ *
+ * Since: 2.91.1
+ **/
+const GcmColorYxy *
+gcm_edid_get_green (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->green;
+}
+
+/**
+ * gcm_edid_get_blue:
+ * @edid: a valid #GcmEdid instance
+ *
+ * Gets the monitor red chromaticity value.
+ *
+ * Return value: the #GcmColorYxy value
+ *
+ * Since: 2.91.1
+ **/
+const GcmColorYxy *
+gcm_edid_get_blue (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->blue;
+}
+
+/**
+ * gcm_edid_get_white:
+ * @edid: a valid #GcmEdid instance
+ *
+ * Gets the monitor white chromaticity value.
+ *
+ * Return value: the #GcmColorYxy value
+ *
+ * Since: 2.91.1
+ **/
+const GcmColorYxy *
+gcm_edid_get_white (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->white;
+}
+
 /**
  * gcm_edid_reset:
  * @edid: a valid #GcmEdid instance
  *
  * Resets all the cached values in this object.
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 void
 gcm_edid_reset (GcmEdid *edid)
@@ -385,7 +459,7 @@ out:
  *
  * Return value: %TRUE for success.
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 gboolean
 gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
@@ -395,7 +469,6 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
 	GcmEdidPrivate *priv = edid->priv;
 	guint32 serial;
 	guint extension_blocks;
-	gdouble x, y;
 	gchar *tmp;
 
 	g_return_val_if_fail (GCM_IS_EDID (edid), FALSE);
@@ -420,7 +493,7 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
 	priv->pnp_id[0] = 'A' + ((data[GCM_EDID_OFFSET_PNPID+0] & 0x7c) / 4) - 1;
 	priv->pnp_id[1] = 'A' + ((data[GCM_EDID_OFFSET_PNPID+0] & 0x3) * 8) + ((data[GCM_EDID_OFFSET_PNPID+1] & 0xe0) / 32) - 1;
 	priv->pnp_id[2] = 'A' + (data[GCM_EDID_OFFSET_PNPID+1] & 0x1f) - 1;
-	egg_debug ("PNPID: %s", priv->pnp_id);
+	g_debug ("PNPID: %s", priv->pnp_id);
 
 	/* maybe there isn't a ASCII serial number descriptor, so use this instead */
 	serial = (guint32) data[GCM_EDID_OFFSET_SERIAL+0];
@@ -429,7 +502,7 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
 	serial += (guint32) data[GCM_EDID_OFFSET_SERIAL+3] * 0x1000000;
 	if (serial > 0) {
 		priv->serial_number = g_strdup_printf ("%" G_GUINT32_FORMAT, serial);
-		egg_debug ("Serial: %s", priv->serial_number);
+		g_debug ("Serial: %s", priv->serial_number);
 	}
 
 	/* get the size */
@@ -445,31 +518,31 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
 	/* get gamma */
 	if (data[GCM_EDID_OFFSET_GAMMA] == 0xff) {
 		priv->gamma = 1.0f;
-		egg_debug ("gamma is stored in an extension block");
+		g_debug ("gamma is stored in an extension block");
 	} else {
 		priv->gamma = ((gfloat) data[GCM_EDID_OFFSET_GAMMA] / 100) + 1;
-		egg_debug ("gamma is reported as %f", priv->gamma);
+		g_debug ("gamma is reported as %f", priv->gamma);
 	}
 
 	/* get color red */
-	x = gcm_edid_decode_fraction (data[0x1b], gcm_edid_get_bits (data[0x19], 6, 7));
-	y = gcm_edid_decode_fraction (data[0x1c], gcm_edid_get_bits (data[0x19], 5, 4));
-	egg_debug ("red x=%f,y=%f", x, y);
+	priv->red->x = gcm_edid_decode_fraction (data[0x1b], gcm_edid_get_bits (data[0x19], 6, 7));
+	priv->red->y = gcm_edid_decode_fraction (data[0x1c], gcm_edid_get_bits (data[0x19], 5, 4));
+	g_debug ("red x=%f,y=%f", priv->red->x, priv->red->y);
 
 	/* get color green */
-	x = gcm_edid_decode_fraction (data[0x1d], gcm_edid_get_bits (data[0x19], 2, 3));
-	y = gcm_edid_decode_fraction (data[0x1e], gcm_edid_get_bits (data[0x19], 0, 1));
-	egg_debug ("green x=%f,y=%f", x, y);
+	priv->green->x = gcm_edid_decode_fraction (data[0x1d], gcm_edid_get_bits (data[0x19], 2, 3));
+	priv->green->y = gcm_edid_decode_fraction (data[0x1e], gcm_edid_get_bits (data[0x19], 0, 1));
+	g_debug ("green x=%f,y=%f", priv->green->x, priv->green->y);
 
 	/* get color blue */
-	x = gcm_edid_decode_fraction (data[0x1f], gcm_edid_get_bits (data[0x1a], 6, 7));
-	y = gcm_edid_decode_fraction (data[0x20], gcm_edid_get_bits (data[0x1a], 4, 5));
-	egg_debug ("blue x=%f,y=%f", x, y);
+	priv->blue->x = gcm_edid_decode_fraction (data[0x1f], gcm_edid_get_bits (data[0x1a], 6, 7));
+	priv->blue->y = gcm_edid_decode_fraction (data[0x20], gcm_edid_get_bits (data[0x1a], 4, 5));
+	g_debug ("blue x=%f,y=%f", priv->blue->x, priv->blue->y);
 
 	/* get color white */
-	x = gcm_edid_decode_fraction (data[0x21], gcm_edid_get_bits (data[0x1a], 2, 3));
-	y = gcm_edid_decode_fraction (data[0x22], gcm_edid_get_bits (data[0x1a], 0, 1));
-	egg_debug ("white x=%f,y=%f", x, y);
+	priv->white->x = gcm_edid_decode_fraction (data[0x21], gcm_edid_get_bits (data[0x1a], 2, 3));
+	priv->white->y = gcm_edid_decode_fraction (data[0x22], gcm_edid_get_bits (data[0x1a], 0, 1));
+	g_debug ("white x=%f,y=%f", priv->white->x, priv->white->y);
 
 	/* parse EDID data */
 	for (i=GCM_EDID_OFFSET_DATA_BLOCKS; i <= GCM_EDID_OFFSET_LAST_BLOCK; i+=18) {
@@ -493,7 +566,7 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
 				priv->serial_number = tmp;
 			}
 		} else if (data[i+3] == GCM_DESCRIPTOR_COLOR_MANAGEMENT_DATA) {
-			egg_warning ("failing to parse color management data");
+			g_warning ("failing to parse color management data");
 		} else if (data[i+3] == GCM_DESCRIPTOR_ALPHANUMERIC_DATA_STRING) {
 			tmp = gcm_edid_parse_string (&data[i+5]);
 			if (tmp != NULL) {
@@ -502,14 +575,14 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
 			}
 		} else if (data[i+3] == GCM_DESCRIPTOR_COLOR_POINT) {
 			if (data[i+3+9] != 0xff) {
-				egg_debug ("extended EDID block(1) which contains a better gamma value");
+				g_debug ("extended EDID block(1) which contains a better gamma value");
 				priv->gamma = ((gfloat) data[i+3+9] / 100) + 1;
-				egg_debug ("gamma is overridden as %f", priv->gamma);
+				g_debug ("gamma is overridden as %f", priv->gamma);
 			}
 			if (data[i+3+14] != 0xff) {
-				egg_debug ("extended EDID block(2) which contains a better gamma value");
+				g_debug ("extended EDID block(2) which contains a better gamma value");
 				priv->gamma = ((gfloat) data[i+3+9] / 100) + 1;
-				egg_debug ("gamma is overridden as %f", priv->gamma);
+				g_debug ("gamma is overridden as %f", priv->gamma);
 			}
 		}
 	}
@@ -517,16 +590,16 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, gsize length, GError **error)
 	/* extension blocks */
 	extension_blocks = data[GCM_EDID_OFFSET_EXTENSION_BLOCK_COUNT];
 	if (extension_blocks > 0)
-		egg_warning ("%i extension blocks to parse", extension_blocks);
+		g_warning ("%i extension blocks to parse", extension_blocks);
 
 	/* calculate checksum */
 	priv->checksum = g_compute_checksum_for_data (G_CHECKSUM_MD5, data, 0x6c);
 
 	/* print what we've got */
-	egg_debug ("monitor name: %s", priv->monitor_name);
-	egg_debug ("serial number: %s", priv->serial_number);
-	egg_debug ("ascii string: %s", priv->eisa_id);
-	egg_debug ("checksum: %s", priv->checksum);
+	g_debug ("monitor name: %s", priv->monitor_name);
+	g_debug ("serial number: %s", priv->serial_number);
+	g_debug ("ascii string: %s", priv->eisa_id);
+	g_debug ("checksum: %s", priv->checksum);
 out:
 	return ret;
 }
@@ -568,6 +641,18 @@ gcm_edid_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
 	case PROP_HEIGHT:
 		g_value_set_uint (value, priv->height);
 		break;
+	case PROP_WHITE:
+		g_value_set_boxed (value, g_boxed_copy (GCM_TYPE_COLOR_XYZ, priv->white));
+		break;
+	case PROP_RED:
+		g_value_set_boxed (value, g_boxed_copy (GCM_TYPE_COLOR_XYZ, priv->red));
+		break;
+	case PROP_GREEN:
+		g_value_set_boxed (value, g_boxed_copy (GCM_TYPE_COLOR_XYZ, priv->green));
+		break;
+	case PROP_BLUE:
+		g_value_set_boxed (value, g_boxed_copy (GCM_TYPE_COLOR_XYZ, priv->blue));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -671,6 +756,39 @@ gcm_edid_class_init (GcmEdidClass *klass)
 				   G_PARAM_READABLE);
 	g_object_class_install_property (object_class, PROP_HEIGHT, pspec);
 
+	/**
+	 * GcmEdid:red:
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_RED,
+					 g_param_spec_boxed ("red", NULL, NULL,
+							     GCM_TYPE_COLOR_YXY,
+							     G_PARAM_READABLE));
+	/**
+	 * GcmEdid:green:
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_GREEN,
+					 g_param_spec_boxed ("green", NULL, NULL,
+							     GCM_TYPE_COLOR_YXY,
+							     G_PARAM_READABLE));
+	/**
+	 * GcmEdid:blue:
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_BLUE,
+					 g_param_spec_boxed ("blue", NULL, NULL,
+							     GCM_TYPE_COLOR_YXY,
+							     G_PARAM_READABLE));
+	/**
+	 * GcmEdid:white:
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_WHITE,
+					 g_param_spec_boxed ("white", NULL, NULL,
+							     GCM_TYPE_COLOR_YXY,
+							     G_PARAM_READABLE));
+
 	g_type_class_add_private (klass, sizeof (GcmEdidPrivate));
 }
 
@@ -688,6 +806,10 @@ gcm_edid_init (GcmEdid *edid)
 	edid->priv->checksum = NULL;
 	edid->priv->tables = gcm_tables_new ();
 	edid->priv->pnp_id = g_new0 (gchar, 4);
+	edid->priv->red = gcm_color_new_Yxy ();
+	edid->priv->green = gcm_color_new_Yxy ();
+	edid->priv->blue = gcm_color_new_Yxy ();
+	edid->priv->white = gcm_color_new_Yxy ();
 }
 
 /**
@@ -705,6 +827,10 @@ gcm_edid_finalize (GObject *object)
 	g_free (priv->eisa_id);
 	g_free (priv->checksum);
 	g_free (priv->pnp_id);
+	gcm_color_free_Yxy (priv->white);
+	gcm_color_free_Yxy (priv->red);
+	gcm_color_free_Yxy (priv->green);
+	gcm_color_free_Yxy (priv->blue);
 	g_object_unref (priv->tables);
 
 	G_OBJECT_CLASS (gcm_edid_parent_class)->finalize (object);
@@ -715,7 +841,7 @@ gcm_edid_finalize (GObject *object)
  *
  * Return value: a new #GcmEdid object.
  *
- * Since: 0.0.1
+ * Since: 2.91.1
  **/
 GcmEdid *
 gcm_edid_new (void)
